@@ -1,9 +1,10 @@
-# $Id: TEXT.pm 75 2005-08-22 18:22:43Z chronos $
+# $Id: TEXT.pm 91 2005-08-27 11:00:11Z chronos $
 package BBCode::Tag::TEXT;
 use base qw(BBCode::Tag);
 use BBCode::Util qw(encodeHTML);
 use strict;
 use warnings;
+our $VERSION = '0.20';
 
 sub Class($):method {
 	return qw(TEXT INLINE);
@@ -19,7 +20,11 @@ sub DefaultParam($):method {
 
 sub toBBCode($):method {
 	my $this = shift;
-	return $this->param('STR');
+	local $_ = $this->param('STR');
+	s/\[/[]/g;
+	s/&/[ENT=amp]/g;
+	s/<(?=URL:)/[ENT=lt]/gi;
+	return $_;
 }
 
 sub toHTML($):method {
